import type { ColumnInfo, DatabaseInfo, EngineDriver, SqlObjectInfo, TableInfo } from 'dbgate-types';
import { AlterPlan } from './alterPlan';
declare type DbDiffSchemaMode = 'strict' | 'ignore' | 'ignoreImplicit';
export interface DbDiffOptions {
    ignoreCase?: boolean;
    schemaMode?: DbDiffSchemaMode;
    leftImplicitSchema?: string;
    rightImplicitSchema?: string;
    ignoreConstraintNames?: boolean;
    noDropTable?: boolean;
    noDropColumn?: boolean;
    noDropConstraint?: boolean;
    noDropSqlObject?: boolean;
    noRenameTable?: boolean;
    noRenameColumn?: boolean;
    ignoreForeignKeyActions?: boolean;
    ignoreDataTypes?: boolean;
}
export declare function generateTablePairingId(table: TableInfo): TableInfo;
export declare function removeTablePairingId(table: TableInfo): TableInfo;
export declare function generateDbPairingId(db: DatabaseInfo): DatabaseInfo;
export declare function testEqualColumns(a: ColumnInfo, b: ColumnInfo, checkName: boolean, checkDefault: boolean, opts?: DbDiffOptions): boolean;
export declare function testEqualTypes(a: ColumnInfo, b: ColumnInfo, opts?: DbDiffOptions): boolean;
export declare function testEqualTables(a: TableInfo, b: TableInfo, opts: DbDiffOptions, wholeOldDb: DatabaseInfo, wholeNewDb: DatabaseInfo, driver: EngineDriver): boolean;
export declare function testEqualSqlObjects(a: SqlObjectInfo, b: SqlObjectInfo, opts: DbDiffOptions): boolean;
export declare function createAlterTablePlan(oldTable: TableInfo, newTable: TableInfo, opts: DbDiffOptions, wholeOldDb: DatabaseInfo, wholeNewDb: DatabaseInfo, driver: EngineDriver): AlterPlan;
export declare function createAlterDatabasePlan(oldDb: DatabaseInfo, newDb: DatabaseInfo, opts: DbDiffOptions, wholeOldDb: DatabaseInfo, wholeNewDb: DatabaseInfo, driver: EngineDriver): AlterPlan;
export declare function getAlterTableScript(oldTable: TableInfo, newTable: TableInfo, opts: DbDiffOptions, wholeOldDb: DatabaseInfo, wholeNewDb: DatabaseInfo, driver: EngineDriver): {
    sql: string;
    recreates: any[];
} | {
    sql: string;
    recreates: {
        tables: number;
        constraints: number;
        sqlObjects: number;
    };
};
export declare function getAlterDatabaseScript(oldDb: DatabaseInfo, newDb: DatabaseInfo, opts: DbDiffOptions, wholeOldDb: DatabaseInfo, wholeNewDb: DatabaseInfo, driver: EngineDriver): {
    sql: string;
    recreates: {
        tables: number;
        constraints: number;
        sqlObjects: number;
    };
    isEmpty: boolean;
};
export declare function matchPairedObjects(db1: DatabaseInfo, db2: DatabaseInfo, opts: DbDiffOptions): DatabaseInfo;
export declare const modelCompareDbDiffOptions: DbDiffOptions;
export {};
